/*
 * Copyright (c) 2012 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic Semiconductor. The use,
 * copying, transfer or disclosure of such information is prohibited except by express written
 * agreement with Nordic Semiconductor.
 *
 */

/*  This module is a modified version of the Cycling Speed and Cadence Service Client provided with the SDK from
 *  Nordic Semiconductors. This module is not initialized with a single client structure, instead
 *  it is initialized with an array of Client Structures. Therefore it is possible to handle more
 *  than one csc client with this module. The handling of the array is done automatically by this
 *  module.
 */

/**@file
 *
 * @defgroup ble_sdk_srv_csc_c   Cycling Speed and Cadence Service Client
 * @{
 * @ingroup  ble_sdk_srv
 * @brief    Cycling Speed and Cadence Service Client module.
 *
 * @details  This module contains the APIs and types exposed by the Cycling Speed and Cadence Service Client
 *           module. These APIs and types can be used by the application to perform discovery of
 *           Cycling Speed and Cadence Service at the peer and interact with it.
 *
 * @warning  Currently this module only has support for Cycling Speed and Cadence Measurement and Feature
 *           characteristic. This means that it will be able to enable notification of the characteristic
 *           at the peer and be able to receive Cycling Speed and Cadence Measurement notifications from
 *           the peer. It is also able to read the feature characteristic. It does not support the Sensor
 *           Location and the Cycling Speed and Cadence Control Point characteristics.
 *           When a Cycling Speed and Cadence Measurement is received, this module will decode wheel
 *           revolutions, wheel event time, cumulative crank revolutions and crank event time.
 *
 * @note     The application must propagate BLE stack events to this module by calling
 *           ble_csc_c_on_ble_evt().
 *
 */

#ifndef BLE_CSC_C_H__
#define BLE_CSC_C_H__

#include <stdbool.h>
#include <stdint.h>
#include "ble.h"

/**
 * @defgroup csc_c_enums Enumerations
 * @{
 */

/**@brief CSC Client event type. */
typedef enum
{
    BLE_CSC_C_EVT_DISCOVERY_COMPLETE = 1,  /**< Event indicating that the Cycling Speed and Cadence Service has been discovered at the peer. */
    BLE_CSC_C_EVT_CSCM_NOTIFICATION,       /**< Event indicating that a notification of the Cycling Speed and Cadence Measurement characteristic has been received from the peer. */
    BLE_CSC_C_EVT_CSCF_READ_RESP           /**< Event indicating that a read response of the Cycling Speed and Cadence Feature characteristic has bees received. */
} ble_csc_c_evt_type_t;

/** @} */

/**
 * @defgroup CSC_c_structs Structures
 * @{
 */

/**@brief Structure containing the Cycling Speed and Cadence measurement received from the peer. */
typedef struct
{
    uint8_t     wheel_present:1;    /**< Wheel Data present */
    uint8_t     crank_present:1;    /**< Crank Data present */
    uint32_t    wheel_revolutions;  /**< Cumulative Wheel Revolutions. */
    uint16_t    wheel_event_time;   /**< Last Wheel Event Time */
    uint16_t    crank_revolutions;  /**< Cumulative Crank Revolutions */
    uint16_t    crank_event_time;   /**< Last Crank Event Time */
} ble_cscm_t;

/**@brief Structure containing the Cycling Speed and Cadence Feature received from the peer. */
typedef struct
{
    uint16_t    wheel_revo_supported:1; /**< Wheel Revolution Data Supported */
    uint16_t    crank_revo_supported:1; /**< Crank Revolution Data Supported */
    uint16_t    multiple_sensor_loc :1; /**< Multiple Sensor Locations Supported */
} ble_cscf_t;

/**@brief Cycling Speed and Cadence Event structure. */
typedef struct
{
    ble_csc_c_evt_type_t evt_type;  /**< Type of the event. */
    union
    {
        ble_cscm_t cscm;  /**< CSC measurement received. This will be filled if the evt_type is @ref BLE_CSC_C_EVT_CSCM_NOTIFICATION. */
        ble_cscf_t cscf;  /**< CSC feature received. This will be filled if the evt_type is @ref BLE_CSC_C_EVT_CSCF_READ_RESP. */
    } params;
} ble_csc_c_evt_t;

/** @} */

/**
 * @defgroup hrs_c_types Types
 * @{
 */

// Forward declaration of the ble_csc_t type.
typedef struct ble_csc_c_s ble_csc_c_t;

/**@brief   Event handler type.
 *
 * @details This is the type of the event handler that should be provided by the application
 *          of this module in order to receive events.
 */
typedef void (* ble_csc_c_evt_handler_t) (ble_csc_c_t * p_ble_csc_c, ble_csc_c_evt_t * p_evt);

/** @} */

/**
 * @addtogroup hrs_c_structs
 * @{
 */

/**@brief Cycling Speed and Cadence Client structure.
 */
struct ble_csc_c_s
{
    uint16_t                conn_handle;      /**< Connection handle as provided by the SoftDevice. */
    uint16_t                cscm_cccd_handle; /**< Handle of the CCCD of the Cycling Speed and Cadence Measurement characteristic. */
    uint16_t                cscm_handle;      /**< Handle of the Cycling Speed and Cadence Measurement characteristic as provided by the SoftDevice. */
    uint16_t                cscf_handle;      /**< Handle of the Cycling Speed and Cadence feature characteristic as provided by the SoftDevice. */
    ble_csc_c_evt_handler_t evt_handler;      /**< Application event handler to be called when there is an event related to the heart rate service. */
};

/**@brief Cycling Speed and Cadence Client initialization structure.
 */
typedef struct
{
    ble_csc_c_evt_handler_t evt_handler;  /**< Event handler to be called by the Cycling Speed and Cadence Client module whenever there is an event related to the Cycling Speed and Cadence Service. */
} ble_csc_c_init_t;

/** @} */

/**
 * @defgroup hrs_c_functions Functions
 * @{
 */

/**@brief     Function for initializing the cycling speed and Cadence client module.
 *
 * @details   This function will register with the DB Discovery module. There it
 *            registers for the Cycling Speed and Cadence Service. Doing so will make the DB Discovery
 *            module look for the presence of a Cycling Speed and Cadence Service instance at the peer when a
 *            discovery is started.
 *
 * @param[in] p_ble_csc_c      Pointer to an array of cycling speed and cadence client structures.
 * @param[in] num_csc_c        Number of cycling speed and cadence client structures in array
 * @param[in] p_ble_csc_c_init Pointer to the cycling speed and cadence initialization structure containing the
 *                             initialization information.
 *
 * @retval    NRF_SUCCESS On successful initialization. Otherwise an error code. This function
 *                        propagates the error code returned by the Database Discovery module API
 *                        @ref ble_db_discovery_evt_register.
 */
uint32_t ble_csc_c_init(ble_csc_c_t * p_ble_csc_c, uint8_t num_csc_c, ble_csc_c_init_t * p_ble_csc_c_init);

/**@brief     Function for handling BLE events from the SoftDevice.
 *
 * @details   This function will handle the BLE events received from the SoftDevice. If a BLE
 *            event is relevant to the Cycling Speed and Cadence Client module, then it uses it to update
 *            interval variables and, if necessary, send events to the application.
 *
 * @param[in] p_ble_evt   Pointer to the BLE event.
 */
void ble_csc_c_on_ble_evt(const ble_evt_t * p_ble_evt);


/**@brief   Function for requesting the peer to start sending notification of Cycling Speed and Cadence
 *          Measurement.
 *
 * @details This function will enable to notification of the Cycling Speed and Cadence Measurement at the peer
 *          by writing to the CCCD of the Cycling Speed and Cadence Measurement Characteristic.
 *
 * @param   conn_handle handle of the device.
 * @param   enable      true to enable notification, false to disable.
 *
 * @retval  NRF_SUCCESS If the SoftDevice has been requested to write to the CCCD of the peer.
 *                      Otherwise, an error code. This function propagates the error code returned
 *                      by the SoftDevice API @ref sd_ble_gattc_write.
 */
uint32_t ble_csc_c_cscm_notif_enable(uint16_t conn_handle, bool enable);


/**@brief   Function for read the Cycling Speed and Cadence Feature characteristic.
 *
 * @param   conn_handle handle of the device.
 *
 * @retval  NRF_SUCCESS     If the read request was successfully queued to be sent to peer.
 *                          Otherwise, an error code returned by the SoftDevice API @ref
 *                          sd_ble_gattc_write.
 */
uint32_t ble_csc_c_cscf_read(uint16_t conn_handle);

/** @} */ // End tag for Function group.

#endif // BLE_HRS_C_H__

/** @} */ // End tag for the file.
